import ee


SPECTRAL_INDICES_URL = "https://raw.githubusercontent.com/awesome-spectral-indices/awesome-spectral-indices/main/output/spectral-indices-dict.json"
CONSTANTS_URL = "https://raw.githubusercontent.com/awesome-spectral-indices/awesome-spectral-indices/main/output/constants.json"

DATASET_IDS = {
    'L9': "LANDSAT/LC09/C02/T1_L2", 
    'L8': "LANDSAT/LC08/C02/T1_L2", 
    'L7': "LANDSAT/LE07/C02/T1_L2",
    'L5': "LANDSAT/LT05/C02/T1_L2",
    'MOD09A1': "MODIS/061/MOD09A1",
    'MCD43A4': "MODIS/061/MCD43A4"
}

ORIGINAL_BANDS = {
    'L9': ['SR_B1', 'SR_B2', 'SR_B3', 'SR_B4', 'SR_B5', 'SR_B6', 'SR_B7'], 
    'L8': ['SR_B1', 'SR_B2', 'SR_B3', 'SR_B4', 'SR_B5', 'SR_B6', 'SR_B7'], 
    'L7': ['SR_B1', 'SR_B2', 'SR_B3', 'SR_B4', 'SR_B5', 'SR_B7'],
    'L5': ['SR_B1', 'SR_B2', 'SR_B3', 'SR_B4', 'SR_B5', 'SR_B7'],
    'MOD09A1': ['sur_refl_b01', 'sur_refl_b02', 'sur_refl_b03', 'sur_refl_b04', 'sur_refl_b05', 'sur_refl_b06', 'sur_refl_b07'],
    'MCD43A4': ['Nadir_Reflectance_Band1', 'Nadir_Reflectance_Band2', 'Nadir_Reflectance_Band3', 'Nadir_Reflectance_Band4', 'Nadir_Reflectance_Band5', 'Nadir_Reflectance_Band6', 'Nadir_Reflectance_Band7']
}

RENAMED_BANDS = {
    'L9': ['ub', 'blue', 'green', 'red', 'nir', 'swir1', 'swir2'], 
    'L8': ['ub', 'blue', 'green', 'red', 'nir', 'swir1', 'swir2'], 
    'L7': ['blue', 'green', 'red', 'nir', 'swir1', 'swir2'],
    'L5': ['blue', 'green', 'red', 'nir', 'swir1', 'swir2'],
    'MOD09A1': ['red', 'nir', 'blue', 'green', 'mir', 'swir1', 'swir2'],
    'MCD43A4': ['red', 'nir', 'blue', 'green', 'mir', 'swir1', 'swir2']
}


BAND_MAPPING = {
    'B': 'blue',
    'G': 'green',
    'R': 'red',
    'N': 'nir',
    'S1': 'swir1',
    'S2': 'swir2',
    'M': 'mir',
    'UB': 'ub'
}



# 新增的输入验证函数
def validate_inputs(date_range, roi):
    """
    验证用户输入的参数是否合法。
    """
    if not isinstance(date_range, list) or len(date_range) != 2:
        raise ValueError("date_range must be a list with two elements: [start_date, end_date].")
    if not isinstance(roi, ee.Geometry):
        raise ValueError("roi must be an ee.Geometry object.")



__all__ = [
    "DATASET_IDS", 
    "ORIGINAL_BANDS", 
    "RENAMED_BANDS", 
    "BAND_MAPPING", 
    "SPECTRAL_INDICES_URL", 
    "CONSTANTS_URL",
    "validate_inputs"
]


import ee
import json
import urllib.request
from .params import *
from .utils import *

def rm_landsat_cloud(image):
    """
    对 Landsat 影像应用云掩膜处理。

    Args:
        image (ee.Image): 输入的 Landsat 影像。
    
    Returns:
        ee.Image: 经过云掩膜处理的影像，移除了云和云影像素。
    """
    qa = image.select('QA_PIXEL')
    cloud_mask = qa.bitwiseAnd(1 << 3).eq(0)  # 云标志
    cloud_shadow_mask = qa.bitwiseAnd(1 << 4).eq(0)  # 云影标志
    return image.updateMask(cloud_mask.And(cloud_shadow_mask))


def rm_modis_cloud(image):
    """
    对 MODIS 影像应用严格的云掩膜处理。

    Args:
        image (ee.Image): 输入的 MODIS 影像。
    
    Returns:
        ee.Image: 经过云掩膜处理的影像，移除了云和云影像素。
    """
    qa = image.select('StateQA')
    # 清晰像素（Bits 0-1）
    cloud_state_clear = qa.bitwiseAnd(3).eq(0)
    # 无云影（Bit 2）
    no_cloud_shadow = qa.bitwiseAnd(1 << 2).eq(0)
    # 无内部云（Bit 10）
    no_internal_cloud = qa.bitwiseAnd(1 << 10).eq(0)
    return image.updateMask(cloud_state_clear.And(no_cloud_shadow).And(no_internal_cloud))


def rm_MCD43A4_cloud(image):
    """
    Apply quality control (QC) mask to the MCD43A4 image for all 7 bands using the quality bands.
    
    Args:
        image (ee.Image): Input MODIS MCD43A4 image.
    
    Returns:
        ee.Image: Image with bad quality pixels masked out for all 7 bands.
    """
    # Extract the quality control bands for each of the 7 bands
    qc_band1 = image.select('BRDF_Albedo_Band_Mandatory_Quality_Band1')
    qc_band2 = image.select('BRDF_Albedo_Band_Mandatory_Quality_Band2')
    qc_band3 = image.select('BRDF_Albedo_Band_Mandatory_Quality_Band3')
    qc_band4 = image.select('BRDF_Albedo_Band_Mandatory_Quality_Band4')
    qc_band5 = image.select('BRDF_Albedo_Band_Mandatory_Quality_Band5')
    qc_band6 = image.select('BRDF_Albedo_Band_Mandatory_Quality_Band6')
    qc_band7 = image.select('BRDF_Albedo_Band_Mandatory_Quality_Band7')
    
    # Create a mask for good quality (Bit 0 = 0 means good quality)
    # good_quality_mask = (
    #     qc_band1.eq(0)  # Band 1 good quality
    #     .And(qc_band2.eq(0))  # Band 2 good quality
    #     .And(qc_band3.eq(0))  # Band 3 good quality
    #     .And(qc_band4.eq(0))  # Band 4 good quality
    #     .And(qc_band5.eq(0))  # Band 5 good quality
    #     .And(qc_band6.eq(0))  # Band 6 good quality
    #     .And(qc_band7.eq(0))  # Band 7 good quality
    # )
    good_quality_mask = (
        qc_band2.eq(0)
    )

    
    # Apply the mask to the entire image (only keep good quality pixels)
    image = image.updateMask(good_quality_mask)
    
    return image



def add_spectral_indices(image, indices, keep_original=True):
    """
    为单张影像计算并添加指定的光谱指数。

    Args:
        image (ee.Image): 输入影像。
        indices (list): 要计算的光谱指数列表（如 ["NDVI", "EVI"]）。
        keep_original (bool): 是否保留原始影像波段。
            - True: 返回包含原始波段和计算的指数。
            - False: 返回仅包含计算的指数。

    Returns:
        ee.Image: 包含计算结果的影像。
    """
    spectral_indices = fetch_json(SPECTRAL_INDICES_URL)["SpectralIndices"]
    constants = fetch_json(CONSTANTS_URL)
    constant_values = {key: value["default"] for key, value in constants.items() if value["default"] is not None}
    result_image = image

    for index in indices:
        if index not in spectral_indices:
            raise ValueError(f"指数 {index} 不存在于 JSON 文件中。")

        formula = spectral_indices[index]["formula"]
        bands = spectral_indices[index]["bands"]

        # 构建公式的参数字典：映射波段或常数
        params = {
            symbol: image.select(BAND_MAPPING[symbol]) if symbol in BAND_MAPPING
            else image.constant(constant_values[symbol]) for symbol in bands
        }

        # 添加计算结果到影像
        result_image = result_image.addBands(image.expression(formula, params).rename(index))

    return result_image if keep_original else result_image.select(indices)


def add_spectral_indices_to_collection(imgcol, indices, keep_original=True):
    """
    为影像集合中的每个影像计算并添加指定的光谱指数。

    Args:
        imgcol (ee.ImageCollection): 输入的影像集合。
        indices (list): 要计算的光谱指数列表（如 ["NDVI", "EVI"]）。
        keep_original (bool): 是否保留原始影像波段。
            - True: 返回包含原始波段和计算的指数的集合。
            - False: 返回仅包含计算的指数的集合。

    Returns:
        ee.ImageCollection: 包含计算结果的影像集合。
    """
    return imgcol.map(lambda img: add_spectral_indices(img, indices, keep_original))


def calculate_terrain_features(region, resolution, resample_method='bilinear'):
    """
    计算给定区域的地形特征，包括海拔、高程和坡度。

    参数:
    region (ee.Geometry): 感兴趣区域。
    resolution (int): 重采样的目标分辨率。
    resample_method (str): 使用的重采样方法。默认是 'bilinear'。

    返回:
    ee.Image: 包含地形特征（海拔、高程、坡度）的图像，重采样到指定分辨率。
    """
    # 加载SRTM数据（30米分辨率）
    srtm = ee.Image('USGS/SRTMGL1_003')
    
    # 计算坡度和高程
    slope = ee.Terrain.slope(srtm)
    aspect = ee.Terrain.aspect(srtm)
    
    # 将地形特征（海拔、高程、坡度）组合成一个图像
    terrain_image = srtm.addBands(slope).addBands(aspect).rename(['elevation', 'slope', 'aspect'])
    
    # 重采样到指定分辨率
    terrain_image_resampled = terrain_image.resample(resample_method)  # 选择重采样方法
    terrain_image_resampled = terrain_image_resampled.reproject(
        crs=srtm.projection(),
        scale=resolution  # 设置目标分辨率
    ).clip(region)  # 裁剪到指定区域
    
    return terrain_image_resampled


def merge_images(collection, interval, aggregation_method='median'):
    """
    对时序影像集合进行每 interval 天的合成，可选择合成方法（均值、中值等）。
    
    Args:
        collection (ee.ImageCollection): 时序影像集合。
        interval (int): 合成时间间隔（天）。
        aggregation_method (str): 合成方法，'mean' 或 'median'。默认 'median'。
        
    Returns:
        ee.ImageCollection: 合成后的影像集合。
    """
    start_date = ee.Date(collection.sort('system:time_start').first().get('system:time_start'))
    end_date = ee.Date(collection.sort('system:time_start').aggregate_max('system:time_start'))
    time_range = ee.List.sequence(start_date.millis(), end_date.millis(), interval * 24 * 60 * 60 * 1000)

    def merge_function(start_millis):
        start = ee.Date(start_millis)
        end = start.advance(interval, 'day')
        subset = collection.filterDate(start, end)
        
        if aggregation_method == 'mean':
            aggregated_image = subset.mean()
        elif aggregation_method == 'median':
            aggregated_image = subset.median()
        elif aggregation_method == 'min':
            aggregated_image = subset.min()  
        elif aggregation_method == 'max':
            aggregated_image = subset.max()
        else:
            raise ValueError(f"Unsupported aggregation method: {aggregation_method}")
        
        return aggregated_image.set('system:time_start', start.millis())

    merged_collection = time_range.map(merge_function)

    return ee.ImageCollection(merged_collection)




__all__ = [
    "rm_landsat_cloud",
    "rm_modis_cloud",
    "rm_MCD43A4_cloud",
    "add_spectral_indices",
    "add_spectral_indices_to_collection",
    "calculate_terrain_features",
    "merge_images"
]


import ee
from .params import *
from .processing import *

def get_any_year_data(date_range, roi, dataset='Landsat', remove_cloud=True, normalize=True, bands=None, landsat_series=None):
    """
    获取指定时间范围和区域的影像集合（支持 Landsat 和 MODIS 数据）。

    Args:
        date_range (list): 包含起始和结束日期的列表，例如 ['2020-01-01', '2020-12-31']。
        roi (ee.Geometry): 感兴趣区域 (Region of Interest)。
        dataset (str): 数据类型（'Landsat' 或 'MODIS'），默认为 'Landsat'。
        remove_cloud (bool): 是否对影像进行云掩膜处理，默认 True。
        normalize (bool): 是否对影像进行归一化处理，默认 True。
        bands (list): 用户自定义的波段选择，默认为 None，表示使用默认波段。
        landsat_series (list): 指定 Landsat 系列，例如 ['L8', 'L9']，默认全部系列 ['L5', 'L7', 'L8', 'L9']。

    Returns:
        ee.ImageCollection: 处理后的影像集合。
    """
    validate_inputs(date_range, roi)

    # 设置默认参数
    if dataset == 'Landsat':
        bands = bands or ['blue', 'green', 'red', 'nir', 'swir1', 'swir2']
        landsat_series = landsat_series or ['L5', 'L7', 'L8', 'L9']
        invalid_series = [s for s in landsat_series if s not in DATASET_IDS]
        if invalid_series:
            raise ValueError(f"Invalid Landsat series: {invalid_series}")
        cloud_function = rm_landsat_cloud
    elif dataset == 'MODIS':
        bands = bands or ['red', 'nir', 'blue', 'green', 'mir', 'swir1', 'swir2']
        cloud_function = rm_modis_cloud
    else:
        raise ValueError(f"Unsupported dataset: {dataset}")

    def process_series(series):
        """
        处理单个数据系列。
        """
        collection = (ee.ImageCollection(DATASET_IDS[series])
                      .filterBounds(roi)
                      .filterDate(date_range[0], date_range[1]))
        if remove_cloud:
            collection = collection.map(cloud_function)

        def process_image(img):
            """
            对影像进行选择、归一化，并保留系统属性。
            """
            img = img.select(ORIGINAL_BANDS[series], RENAMED_BANDS[series]).select(bands)
            if normalize:
                if dataset == 'Landsat':
                    img = img.multiply(0.0000275).add(-0.2)
                elif dataset == 'MODIS':
                    img = img.multiply(0.0001)
            return img.copyProperties(img, ['system:time_start', 'system:time_end', 'system:index'])

        return collection.map(process_image)

    if dataset == 'MODIS':
        return process_series('MOD09A1')

    if dataset == 'Landsat':
        collections = [process_series(series) for series in landsat_series]
        merged_collection = ee.ImageCollection(collections).flatten().sort('system:time_start')
        return merged_collection


def load_modis_data(date_range, roi, series='MOD09A1', remove_cloud=True, normalize=True):
    """
    加载并处理 MODIS 数据集（MOD09A1 或 MCD43A4），包括云掩膜、归一化等处理。

    Args:
        roi (ee.Geometry): 感兴趣区域。
        date_range (list): 日期范围，如 ['2020-01-01', '2020-12-31']。
        series (str): 数据系列，'MOD09A1' 或 'MCD43A4'。
        normalize (bool): 是否进行归一化，默认为 True。
        remove_cloud (bool): 是否去云，默认为 True。

    Returns:
        ee.ImageCollection: 处理后的 MODIS 数据集合。
    """

    # 根据数据系列选择云掩膜函数
    if series == 'MOD09A1':
        cloud_function = rm_modis_cloud  # 假设已经定义了云掩膜函数
    elif series == 'MCD43A4':
        cloud_function = rm_MCD43A4_cloud  # MCD43A4 不需要额外的云掩膜

    def process_series(series):
        """
        处理单个数据系列（加载、过滤、云掩膜、归一化等）。
        """
        # 通过 filterBounds 限制区域，避免加载整个全球数据
        collection = (ee.ImageCollection(DATASET_IDS[series])
                      .filterDate(date_range[0], date_range[1])
                      .filterBounds(roi))  # 需要传入 roi 进行区域限制

        # 如果需要去云，应用云掩膜函数
        if remove_cloud and cloud_function:
            collection = collection.map(cloud_function)

        def process_image(img):
            """
            对单张影像进行波段选择和归一化处理。
            """
            # 选择波段并进行归一化
            img2 = img.select(ORIGINAL_BANDS[series], RENAMED_BANDS[series])
            if normalize:
                img2 = img2.multiply(0.0001)  # 假设 MODIS 数据需要归一化
            # 保留影像的系统属性（如时间信息）
            img2 = img2.copyProperties(img, ['system:time_start', 'system:time_end', 'system:index'])
            return img2

        return collection.map(process_image)

    # 处理 MOD09A1 或 MCD43A4 数据系列
    return process_series(series)


__all__ = ["get_any_year_data",
           "load_modis_data"]


import ee
import importlib
import geepy
from datetime import datetime
from IPython import get_ipython
import urllib.request
import json


#######################################################################
######################################## 日志和执行控制相关
#######################################################################

def log_execution_start(info=None):
    """
    记录代码单元执行的开始时间。

    Args:
        info (str, optional): 可选的附加信息，用于标记该次运行（默认值为 None）。

    Returns:
        datetime: 返回当前的起始时间。
    """
    start_time = datetime.now()
    print(f"\nStart Time: {start_time.strftime('%Y-%m-%d %H:%M:%S')}\n")
    return start_time  # 建议添加返回值方便调试


def register_jupyter_hook():
    """
    注册 Jupyter Notebook 钩子，用于在代码单元运行前自动记录时间。

    Returns:
        bool: 如果成功注册返回 True，否则返回 False。
    """
    ipython = get_ipython()
    if ipython:
        ipython.events.register('pre_run_cell', log_execution_start)
        return True
    else:
        print("当前环境非 Jupyter Notebook，无法注册钩子！")
        return False




#######################################################################
######################################## 动态加载模块
#######################################################################
def reload_package():
    """
    强制重新加载 geepy 包及其所有子模块，适用于开发者模式安装时更新后的动态加载。
    """
    for submodule in list(geepy.__dict__.values()):
        if hasattr(submodule, "__name__") and submodule.__name__.startswith("geepy."):
            importlib.reload(submodule)
    importlib.reload(geepy)




#######################################################################
######################################## 网络资源加载
#######################################################################

def fetch_json(url):
    """
    从指定 URL 加载 JSON 文件。
    
    Args:
        url (str): JSON 文件的 URL。
    
    Returns:
        dict: 解析后的 JSON 数据。
    """
    try:
        with urllib.request.urlopen(url) as response:
            return json.loads(response.read().decode())
    except Exception as e:
        raise RuntimeError(f"无法加载 JSON 文件：{url}\n错误信息：{e}")


#######################################################################
######################################## 网格生成工具
#######################################################################

def generate_rect_grid(study_area, grid_width=1.5, grid_height=1.5):
    """
    在给定的研究区内生成指定大小的矩形网格。

    Args:
        study_area (ee.FeatureCollection): 研究区。
        grid_width (float): 网格宽度（经度方向，单位：度）。
        grid_height (float): 网格高度（纬度方向，单位：度）。

    Returns:
        ee.FeatureCollection: 裁剪后的网格。
    """
    bounds = study_area.bounds()
    coords = bounds.coordinates().get(0)

    min_lon = ee.List(ee.List(coords).get(0)).get(0)
    max_lon = ee.List(ee.List(coords).get(1)).get(0)
    min_lat = ee.List(ee.List(coords).get(0)).get(1)
    max_lat = ee.List(ee.List(coords).get(2)).get(1)

    # 生成经度和纬度列表
    lon_list = ee.List.sequence(min_lon, max_lon, grid_width)
    lat_list = ee.List.sequence(min_lat, max_lat, grid_height)

    # 构建网格
    grid = lon_list.map(
        lambda lon: lat_list.map(
            lambda lat: ee.Feature(
                ee.Geometry.Rectangle(
                    [lon, lat, ee.Number(lon).add(grid_width), ee.Number(lat).add(grid_height)]
                )
            )
        )
    ).flatten()

    # 转换为 FeatureCollection 并裁剪到研究区
    grid_fc = ee.FeatureCollection(grid)
    return grid_fc.filterBounds(study_area)


def generate_hex_grid(study_area, radius=1.5):
    """
    在给定的研究区内生成指定大小的六边形网格

    Args:
        study_area (ee.FeatureCollection): 研究区。
        radius (float): 六边形边到中心的距离（单位：度）。

    Returns:
        ee.FeatureCollection: 裁剪后的六边形网格。
    """
    bounds = study_area.bounds()
    coords = bounds.coordinates().get(0)

    xmin = ee.Number(ee.List(ee.List(coords).get(0)).get(0))
    ymin = ee.Number(ee.List(ee.List(coords).get(0)).get(1))
    xmax = ee.Number(ee.List(ee.List(coords).get(2)).get(0))
    ymax = ee.Number(ee.List(ee.List(coords).get(2)).get(1))

    # 将 Python float 转换为 ee.Number
    radius = ee.Number(radius)
    sqrt_3 = ee.Number(3).sqrt()
    r_half = radius.divide(2)
    r_half_sqrt_3 = r_half.multiply(sqrt_3)
    step_x = radius.multiply(3)
    step_y = radius.multiply(sqrt_3)

    xx1 = ee.List.sequence(xmin, xmax.add(radius), step_x)
    yy1 = ee.List.sequence(ymin, ymax.add(radius), step_y)

    xx2 = ee.List.sequence(xmin.subtract(radius.multiply(1.5)), xmax.add(radius), step_x)
    yy2 = ee.List.sequence(ymin.add(r_half_sqrt_3), ymax.add(radius), step_y)

    def hex(x, y):
        point1_x = ee.Number(x).subtract(radius)
        point2_x = ee.Number(x).subtract(r_half)
        point3_x = ee.Number(x).add(r_half)
        point4_x = ee.Number(x).add(radius)

        point1_y = ee.Number(y).add(r_half_sqrt_3)
        point2_y = ee.Number(y)
        point3_y = ee.Number(y).subtract(r_half_sqrt_3)

        polygon = ee.Geometry.Polygon(
            [[[point1_x, point2_y],
              [point2_x, point3_y],
              [point3_x, point3_y],
              [point4_x, point2_y],
              [point3_x, point1_y],
              [point2_x, point1_y]]]
        )
        return ee.Feature(polygon)

    cell1 = xx1.map(
        lambda x: yy1.map(lambda y: hex(x, y))
    ).flatten()

    cell2 = xx2.map(
        lambda x: yy2.map(lambda y: hex(x, y))
    ).flatten()

    # 将嵌套的列表扁平化并创建 FeatureCollection
    hex_grid = ee.FeatureCollection(cell1.cat(cell2))
    return hex_grid.filterBounds(study_area)


#######################################################################
######################################## 影像处理小工具
#######################################################################

def get_image_collection_dates(imgcol):
    """
    打印影像集合中每个影像的开始时间，并转换为 'yyyy-MM-dd' 的日期格式。

    Args:
        imgcol (ee.ImageCollection): 输入的影像集合。

    Returns:
        list: 包含所有影像开始时间的字符串列表，格式为 'yyyy-MM-dd'。
    """
    def format_start_date(image):
        # 获取影像的开始时间属性并格式化为 'yyyy-MM-dd'
        start_date = ee.Date(image.get('system:time_start')).format('yyyy-MM-dd')
        return ee.Feature(None, {'date': start_date})  # 创建一个包含日期的 Feature

    # 将影像集合映射为日期集合
    date_features = imgcol.map(format_start_date)

    # 提取日期并打印
    dates = date_features.aggregate_array('date').getInfo()
    print("Start dates of images in the collection:")
    # for date in dates:
    #     print(date)

    return dates

# 删除GEE的Folder 和 IMGCOL资产
def delete_asset_folder(asset_path):
    """
    Recursively delete all child assets under a GEE folder or ImageCollection,
    then delete the folder itself.

    Args:
        asset_path (str): Full asset path 
    """
    try:
        children = ee.data.listAssets(asset_path).get('assets', [])
        if not children:
            print(f"No children found in: {asset_path}")
        else:
            for asset in children:
                ee.data.deleteAsset(asset['name'])
                print(f"Deleted: {asset['name']}")
        ee.data.deleteAsset(asset_path)
        print(f"Deleted folder: {asset_path}")
    except Exception as e:
        print(f"Error deleting assets under {asset_path}:\n{e}")


def ensure_asset_exists(asset_path, asset_type='IMAGE_COLLECTION'):
    """Check if an Earth Engine asset exists; create if not."""
    parent = '/'.join(asset_path.split('/')[:-1])
    assets = ee.data.listAssets(parent).get('assets', [])
    existing_names = [a['name'] for a in assets]

    if asset_path in existing_names:
        print(f"Asset exists: {asset_path}")
    else:
        ee.data.createAsset({'type': asset_type}, asset_path)
        print(f"Created asset: {asset_path}")


def copy_imagecollection(src_ic_path, dst_ic_path):
    """
    Copy all images from one Earth Engine ImageCollection to another using ee.data.

    Args:
        src_ic_path (str): Source ImageCollection asset path.
        dst_ic_path (str): Target ImageCollection asset path (must already exist).
    """
    # 创建目标目录（如果不存在）
    ensure_asset_exists(dst_ic_path, asset_type='IMAGE_COLLECTION')

    # 列出源目录下的所有图像
    assets = ee.data.listAssets(src_ic_path).get('assets', [])
    if not assets:
        print(f"No assets found in source: {src_ic_path}")
        return

    for asset in assets:
        if asset['type'] != 'IMAGE':
            continue
        img_name = asset['name'].split('/')[-1]
        src_img_path = asset['name']
        dst_img_path = f"{dst_ic_path}/{img_name}"

        try:
            ee.data.copyAsset(src_img_path, dst_img_path)
            print(f"Copied: {img_name}")
        except Exception as e:
            print(f"Failed to copy {img_name}: {e}")

# 明确需要导出的函数
__all__ = [
    "log_execution_start",
    "register_jupyter_hook",
    "get_image_collection_dates",
    "reload_package",
    "generate_rect_grid",
    "generate_hex_grid",
    "fetch_json", 
    "delete_asset_folder", 
    "ensure_asset_exists", 
    "copy_imagecollection"
]



# geepy/__init__.py



from .gee import (
    image_loader,
    params,
    processing,
    utils,
    visualization
)

# 可选向后兼容：
from .gee.utils import *
from .gee.processing import *
from .gee.image_loader import *
from .gee.params import *
from .gee.visualization import *



from setuptools import setup, find_packages
from pathlib import Path


def get_version():
    version_path = Path(__file__).parent / "geepy" / "__version__.py"
    exec(version_path.read_text(), version := {})
    return version["__version__"]


setup(
    name='geedl',
    version=get_version(),
    description='A Python package for Google Earth Engine tools',
    long_description=open('README.md', encoding='utf-8').read(),
    long_description_content_type='text/markdown',
    author='Zhang Lei',
    author_email='zhanglei1136@163.com',
    url='https://github.com/gg-zl/GEE_py',
    packages=find_packages(), 
    install_requires=[
        'numpy',              
        'earthengine-api',     
        'geemap',             
        'pandas',              
        'matplotlib',           
    ],
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.7',
)


# cd geepy
# change version in setup.py and readme.md
# git add -A
# git commit -m "Bump version to 0.1.2 and add new function xxx"
# git push origin master
# git tag v0.1.2
# git push origin v0.1.2